/*
 * cg_box.scad
 *
 * Written by aubenc @ Thingiverse
 *
 * This script is licensed under the Creative Commons Attribution-ShareAlike license.
 *
 * http://www.thingiverse.com/thing:34021
 *
 * This script is not supposed to be use by itself but to be a complement 
 * of coffee_grinder.scad intended to build the casing for such a thing.
 *
 */

/* *************** Parameters ***************************************************** */
tk=2.5;tk=2.5;
wd=65;
hg=110;
pk=1;
ng=0.1;


/* *************** Computed from above ******************************************** */
ww=wd-2.5;
rd=tk/cos(22.5);


/* *************** Code *********************************************************** */
module box_side()
{
	rounded_frame();
}

module box_bottom()
{
	union()
	{
		rounded_top_bottom();
		translate([0,0,2.5])
		cube(size=[59.6,59.6,5], center=true);
	}
}

module box_top()
{
	difference()
	{
		rounded_top_bottom();

		translate([0,0,-0.1])
		cylinder(h=5, r=21.5/2, $fn=4*21.5*PI,	center=false);
	}
}


module drawer()
{
	union()
	{
		translate([0,0,24.3/2])
		difference()
		{
			cube(size=[54.50,62.5,24.3], center=true);

			translate([0,0,0.7])
			cube(size=[54.50-1.4,62.5-1.4,24.3], center=true);
		}

		translate([0,-(62.5/2-0.5),15.5]) handle();
	}
}

module test_drawer()
{
	translate([0,0,10])
	union()
	{
		difference()
		{
			cube([30,10,20], true);
			translate([0,0,0.8]) cube([30-1.4,10-1.4,20], true);
		}

		translate([0,-4.5,0]) handle();
	}
}

module handle()
{
	translate([0,1,0]) rotate([0,90,0])
	difference()
	{
		union()
		{
			translate([0,0,7.5]) sphere(r=6, $fn=48);
			cylinder(h=15, r=6, $fn=48, center=true);
			translate([0,0,-7.5]) sphere(r=6, $fn=48);
		}

		union()
		{
			translate([0,0,7.5]) sphere(r=5.3, $fn=48);
			cylinder(h=15, r=5.3, $fn=48, center=true);
			translate([0,0,-7.5]) sphere(r=5.3, $fn=48);
		}

		translate([0,4,0]) cube(size=[12,10,40], center=true);

		translate([6.54,-6.54,0]) rotate([0,0,45])
		cube(size=[10,10,50], center=true);
	}
}

module rounded_top_bottom()
{
	intersection()
	{
		rounded_top_bottom_frame();
		top_btt_upsdwn();
	}
}

module rounded_top_bottom_frame()
{
	xy1=rd+0.1;
	lyh=0.2;
	fnn=4*round(2*rd*PI/4/lyh);

	translate([0,0,tk])
	union()
	{
		translate([0,0,-tk/2])
		cube(size=[wd-2*tk,wd-2*tk,tk], center=true);

		for(y=[-1,1])
		{
			translate([0,y*(wd/2-tk),0]) rotate([0,90,0])
			cylinder(h=wd-2*tk, r=rd, $fn=fnn, center=true);
		}

		for(x=[-1,1])
		{
			translate([x*(wd/2-tk),0,0]) rotate([90,0,0])
			cylinder(h=wd-2*tk, r=rd, $fn=fnn, center=true);
		}

		for(x=[-1,1])
		{
			for(y=[-1,1])
			{
				translate([x*(wd/2-tk),y*(wd/2-tk),0])// rotate([90,0,0])
				rotate_extrude(convexity=10, $fn=fnn)
				difference()
				{
					circle(r=rd, fn=fnn);
					polygon(points=[	[-xy1,-xy1],[0,-xy1],[0,xy1],[-xy1,xy1]	],
								paths=[	[0,1,2,3]	]);
				}
			}
		}
	}
}

module top_btt_upsdwn()
{
	translate([0,0,tk]) rotate([180,0,0]) top_bottom_frame();
}

module top_bottom_frame()
{
	difference()
	{
		union()
		{
			translate([0,0,tk/2])
			cube(size=[wd-2*tk,wd-2*tk,tk], center=true);

			for(i=[0:3])
			{
				rotate([0,0,90*i])
				{
					translate([(wd-2*tk+tk)/2, wd/4,0]) notch_3();
					translate([(wd-2*tk+tk)/2,-wd/4,0]) notch_3();
				}
			}
		}

//		translate([0,0,tk/2])
//		cube(size=[wd-2*tk-2*4,wd-2*tk-2*4,tk+0.1], center=true);
	}
}

module rounded_frame()
{
	intersection()
	{
		4x_frame_upsdwn();
		rounded_pannel();
	}
}

module rounded_pannel()
{
	xy1=rd+0.1;
	lyh=0.2;
	fnn=4*round(2*rd*PI/4/lyh);

	translate([0,0,tk])
	union()
	{
		translate([0,0,-tk/2])
		cube(size=[wd-2*tk,hg-2*tk,tk], center=true);

		for(y=[-1,1])
		{
			translate([0,y*(hg/2-tk),0]) rotate([0,90,0])
			cylinder(h=wd-2*tk, r=rd, $fn=fnn, center=true);
		}

		for(x=[-1,1])
		{
			translate([x*(wd/2-tk),0,0]) rotate([90,0,0])
			cylinder(h=hg-2*tk, r=rd, $fn=fnn, center=true);
		}

		for(x=[-1,1])
		{
			for(y=[-1,1])
			{
				translate([x*(wd/2-tk),y*(hg/2-tk),0])// rotate([90,0,0])
				rotate_extrude(convexity=10, $fn=fnn)
				difference()
				{
					circle(r=rd, fn=fnn);
					polygon(points=[	[-xy1,-xy1],[0,-xy1],[0,xy1],[-xy1,xy1]	],
								paths=[	[0,1,2,3]	]);
				}
			}
		}
	}
}

module 4x_frame_upsdwn()
{
	translate([0,0,tk]) rotate([0,180,0]) box_pannel_frame();
}

module box_pannel_frame()
{
	translate([-1.25,0,0])
	difference()
	{
		union()
		{
			translate([0,0,1.25]) cube(size=[ww,hg,2.5], center=true);

			translate([(ww+2.5)/2,45,0]) notch_3();

			translate([(ww+2.5)/2,22.5,0]) notch_1();

			translate([(ww+2.5)/2,0,0]) notch_3();

			translate([(ww+2.5)/2,-22.5,0]) notch_1();

			translate([(ww+2.5)/2,-45,0]) notch_3();
		}

//		translate([1.25,0,pk+tk/2]) cube(size=[ww-4*2+2.5,hg-4*2,tk], center=true);

		translate([wd/4+tk/2,(hg-2.5)/2,0]) rotate([0,0,-90]) notch_2();
		translate([-wd/4+tk/2,(hg-2.5)/2,0]) rotate([0,0,-90]) notch_2();

//		translate([1.25,0,tk/2]) cube(size=[ww-6*2+2.5,hg-6*2,tk+0.2], center=true);

		translate([-(ww-2.5)/2,45,0]) notch_2();

		translate([-(ww-2.5)/2,22.5,0]) notch_4();

		translate([-(ww-2.5)/2,0,0]) notch_2();

		translate([-(ww-2.5)/2,-22.5,0]) notch_4();

		translate([-(ww-2.5)/2,-45,0]) notch_2();

		translate([wd/4+tk/2,-(hg-2.5)/2,0]) rotate([0,0,90]) notch_2();
		translate([-wd/4+tk/2,-(hg-2.5)/2,0]) rotate([0,0,90]) notch_2();
	}
}

module notch_1()
{
	x1=1.25+ng; x2=1.25;
	y1=3-ng; y2=5-ng;

	linear_extrude(height=2.5, convexity=10, center=false)
	polygon(points=[	[-x1,-y1],[-x2,-y1],[x2,-y2],[x2,y2],[-x2,y1],[-x1,y1]	],
				paths=[	[0,1,2,3,4,5]	]);
}

module notch_2()
{
	x1=1.25+ng; x2=1.25;
	y1=3+ng; y2=5+ng;

	translate([0,0,-0.1])
	linear_extrude(height=2.5+0.2, convexity=10, center=false)
	polygon(points=[	[-x1,-y1],[-x2,-y1],[x2,-y2],[x2,y2],[-x2,y1],[-x1,y1]	],
				paths=[	[0,1,2,3,4,5]	]);
}

module notch_3()
{
	x1=1.25;
	y1=3-ng; y2=5-ng;

	translate([-0.1,0,1.25]) rotate([0,90,0])
	linear_extrude(height=2.5+0.2, convexity=10, center=true)
	polygon(points=[	[-x1,-y1],[x1,-y2],[x1,y2],[-x1,y1]	],
				paths=[	[0,1,2,3,4,5]	]);

}

module notch_4()
{
	x1=1.25+ng; x2=1.25;
	y1=3+0.1; y2=5+ng;

	translate([1.25+0.1,0,1.25]) rotate([0,-90,0])
	linear_extrude(height=2.5+0.2, convexity=10, center=false)
	polygon(points=[	[-x1,-y1],[-x2,-y1],[x2,-y2],[x1,-y2],[x1,y2],[x2,y2],[-x2,y1],[-x1,y1]	],
				paths=[	[0,1,2,3,4,5,6,7]	]);
}

