/*
 * coffee_grinder.scad
 *
 * (which in fact is just an overkill pencil sharpener)
 * 
 * Written by aubenc @ Thingiverse
 *
 * This script is licensed under the Creative Commons Attribution-ShareAlike license.
 *
 * http://www.thingiverse.com/thing:34021
 *
 * Requires or uses (reproduced here) code from:
 *
 *	  parametric_involute_gear_v5.0.scad by GregFrost
 *	  http://www.thingiverse.com/thing:3575
 *   Licensed under the Creative commons - LGPL license
 *
 *	  Public Domain Involute Parameterized Gears by LeemonBaird
 *   http://www.thingiverse.com/thing:5505
 *   Licensed under the Public Domain license
 *
 *	  Poor man's openscad screw library by aubenc
 *   http://www.thingiverse.com/thing:8796
 *   Licensed under the Public Domain license
 *
 *	 ...and...
 *
 *    lines();
 *
 *   cg_box.scad by aubenc available together with this thing and
 *   licensed under the Creative Commons Attribution-ShareAlike license
 */


/* *************** Libraries ****************************************************** */
use <parametric_involute_gear_v5.0.scad>
use <polyScrewThread.scad>
use <cg_box.scad>


/* *************** Parameters ***************************************************** */

// Gears and Gears Holder...

gap=0.4;		// Gap between Gears and Holder
wtk=1.2;		// Wall thickness for the Gears Holder
stp=0.2;		// Sticky Pads height, applies only to Gears Holder and Crank Bearing
res=0.5;		// Used "here and there" only
hrd=7.5;		// Radius for the hole in the gears


// Spur Gears...

sgp=32;		// Pitch Diameter
sgt=19;		// Number of Teeth
sgh=4;		// Thickness
sgl=30;		// Pressure Angle


/* *************** Hardcoded ****************************************************** */

scl=1.25;
prd=11.4;
gtk=2.8284;
ghg=16.7778;


/* *************** Computed from above ******************************************** */

sor=hrd-gap/2;
sir=sor-wtk;
sph=prd+gtk/2+1;
sok=sor*cos(45);
sik=sir*cos(45);
gp2=gap/2;
gp4=gp2/2;
fnn=4*round(2*hrd*PI/4/0.5);
dst=2*sph;
ds2=dst/2;
phg=16+gtk*scl;
fst=(110/2-2.5)-(sgp/2+(prd+gtk/2)*scl+8+0.5);


/* *************** Assembly Views ************************************************* */

//	assembly();
//	cross_section();
//	exploded();


/* *************** Modules ******************************************************** */

//	cg_bevel();
//	cg_spur();
//	gear_lock();
//	gears_holder();
//	spacers();
//	bearings(5);
//	crank_bearings(4);

//	sharpener_drum();

//	box_front();
//	box_side();
//	box_back();
//	box_top();
//	box_bottom();
//	drawer();

//	crank_handle();
//	crank_peg();
//	crank_nut();


/* *************** Testing Plates ************************************************* */

//	front_bearings_plate();		// Testing Plate 01
//	2x_pillows();					// Testing Plate 02


/* *************** Code *********************************************************** */
module gears_holder()
{
	s_sph=sph*scl;

	translate([0,0,sok])
	union()
	{
		rotate([-90,0,0]) support_peg_2();

		rotate([0, 90,0]) support_peg_3();
		rotate([0,-90,0]) support_peg_3();

		for(i=[0:2])
		{
			rotate([0,0,i*90])
			if( stp > 0 )
			{
				translate([s_sph+sgh,(sik+sok)/2,-sok])
				cylinder(h=stp, r=2.5, $fn=16, center=false);

				translate([s_sph+sgh,-(sik+sok)/2,-sok])
				cylinder(h=stp, r=2.5, $fn=16, center=false);
			}
		}
	}
}

module support_peg_3()
{
	s_prd=prd*scl;
	s_gtk=gtk*scl;
	s_sph=sph*scl;

	x0=sir;	x1=sor;	x2=sor+0.5;

	y0=-0.1;	y1=s_prd-s_gtk/2-1; y2=y1+0.5;	y3=y2+0.5;	y4=y3+0.5;
	y5=s_prd+s_gtk/2+sgh-0.5;	y6=y5+0.5;	y7=y6+0.5;	y8=y7+0.5;

	difference()
	{
		intersection()
		{
			rotate_extrude(convexity=10, $fn=4*round(2*hrd*PI/4/0.5))
			polygon(points=[	[x0,y0],[x1,y0],[x1,y1],[x2,y2],[x2,y3],[x1,y4],
									[x1,y5],[x2,y6],[x2,y7],[x1,y8],[x0,y8]	],
						paths=[	[0,1,2,3,4,5,6,7,8,9,10]	]);

			translate([0,0,y8/2-0.05])
			cube(size=[2*sok,2*(sor+0.5),y8+0.1], center=true);
		}

		translate([0,0,y8/2-0.05])
		cube(size=[2*(sor+0.5),2*sik,y8+0.1+0.1], center=true);
	}
}

module support_peg_2()
{
	s_prd=prd*scl;
	s_gtk=gtk*scl;
	s_sph=sph*scl;

	x0=sir;	x1=sor;	x2=sor+0.5;

	y0=-(sor+0.1);	y1=s_prd-s_gtk/2-1; y2=y1+0.5;	y3=y2+0.5;	y4=y3+0.5;
	y5=s_prd+s_gtk/2+sgh-0.5;	y6=y5+0.5;	y7=y6+0.5;	y8=y7+0.5;

	difference()
	{
		intersection()
		{
			rotate_extrude(convexity=10, $fn=4*round(2*hrd*PI/4/0.5))
			polygon(points=[	[x0,y0],[x1,y0],[x1,y1],[x2,y2],[x2,y3],[x1,y4],
									[x1,y5],[x2,y6],[x2,y7],[x1,y8],[x0,y8]	],
						paths=[	[0,1,2,3,4,5,6,7,8,9,10]	]);

			union()
			{
				translate([0,0,y8/2-0.05])
				cube(size=[2*(sor+0.5),2*sok,y8+0.1], center=true);

				intersection()
				{
					translate([0,0,0]) rotate([0,90,0])
					cylinder(h=2*(sor+0.5), r=sor-0.1, $fn=8*sor*PI, center=true);

					cube(size=[2*sor,2*sok,2*sor], center=true);
				}
			}
		}

		translate([0,0,y8/2-sor/2-0.05])
		cube(size=[2*sik,2*(sor+0.5),y8+sor+0.1+0.1], center=true);
	}
}

module crank_peg()
{
	union()
	{
		cylinder(h=15, r=9-0.3, $fn=4*9*PI, center=false);

		translate([0,0,9.5])
		cube(size=[10,10,19], center=true);

		intersection()
		{
			for(i=[0:2])
			{
				rotate([0,0,i*120]) translate([7,0,1.3])
				cube(size=[14,4.6,2.6], center=true);
			}

			cylinder(h=2.6, r=14, $fn=4*28*PI, center=false);
		}

		translate([0,0,15])
		difference()
		{
			sphere(r=10.5, $fn=4*9*PI);

			translate([0,0,20]) cube(40, true);
		}

		translate([0,0,19-1.5])
		screw_thread(	10,		// Outer diameter of the thread
						  1.5,		// Step
							50,		// Degrees
						  7.5,		// Length (Z) of the tread
						 1.25,		// Resolution
							 2);		// Top end countersunk

	}
}

module crank_nut()
{
	difference()
	{
		sphere(r=10.5, $fn=4*9*PI);

		translate([0,0,-20]) cube(40, true);

		translate([0,0,-2.25])
		screw_thread(	10.4,		// Outer diameter of the thread
							 1.5,		// Step
							  50,		// Degrees
								9,		// Length (Z) of the tread
							1.25,	// Resolution
								2);		// Top end countersunk
	}
}

module crank_handle()
{
	rd1=21/2;
	rd3=12/2;
	b=rd1-7;
	c=1;
	lf=30;
	bt=45;
	ds=40;

	rd2=(rd1*sin(lf)+b)/(1-sin(lf));
	rd4=(rd3*sin(bt)-c)/(1-sin(bt));

	xc1=(rd1+rd2)*cos(lf);
	yc1=b-rd2;

	xc3=ds-(rd3+rd4)*cos(bt);
	yc3=(21-7+5)/2+c+rd4;

	x0=0;	x1=rd1*cos(lf); x2=xc1;
	y0=rd1; y1=-rd1*sin(lf); y2=b;

	x3=xc3; x4=ds; x5=x4-rd3*cos(bt);
	y3=rd1; y4=(21-7+5)/2+1; y5=y4+rd3*sin(bt)-1;

	cs=sqrt(pow(10+1,2)/2)+0.1;

	difference()
	{
		union()
		{
			cylinder(h=4, r=rd1, $fn=4*round(21*PI/4/0.5), center=false);

			translate([ds/2,(21-7)/2,2])
			cube(size=[ds,7,4], center=true);

			translate([x4,(21-7+5)/2,0])
			cylinder(h=4, r=rd3, $fn=4*round(12*PI/4/0.5), center=false);

			translate([x4,(21-7+5)/2,rd3+1])
			sphere(r=rd3, $fn=4*round(12*PI/4/0.5));

			translate([x4,(21-7+5)/2,rd3+1])
			cylinder(h=10, r=rd3, $fn=4*round(12*PI/4/0.5), center=false);

			translate([x4,(21-7+5)/2,rd3+1+10])
			sphere(r=rd3, $fn=4*round(12*PI/4/0.5));

			linear_extrude(height=4, convexity=10, center=false)
			polygon(points=[[x0,y0],[x1,y1],[x2,y2]], paths=[[0,1,2]]);

			linear_extrude(height=4, convexity=10, center=false)
			polygon(points=[[x3,y3],[x4,y4],[x5,y5]], paths=[[0,1,2]]);
		}
	
		translate([xc1,yc1,-0.1])
		cylinder(h=4+0.2, r=rd2, $fn=4*round(2*rd2*PI/0.5), center=false);

		translate([xc3,yc3,-0.1])
		cylinder(h=4+0.2, r=rd4, $fn=4*round(2*rd4*PI/0.5), center=false);

		if( ds >= 25 )
		{
			translate([21/2+(ds-21/2-12/2)/2,(21-7)/2,2])
			union()
			{
				translate([-(ds-20)/2,0,0])
				cylinder(h=4+0.2, r=1.5, $fn=16, center=true);

				cube(size=[ds-20,3,4+0.2], center=true);

				translate([(ds-20)/2,0,0])
				cylinder(h=4+0.2, r=1.5, $fn=16, center=true);
			}
		}

		translate([0,0,2])
		cube(size=[10.25,10.25,6], center=true);
	}
}

module box_front()
{
	rd=sqrt(pow(4+5.4+1,2)+pow((9.6+0.4)/2+3+0.1,2))+1.5;
	fs=((fst-(16+rd+1.5))+(-110/2))/2;
	dh=abs(2*(fs-(fst-(16+rd+1.5))));

	difference()
	{
		union()
		{
			rounded_frame();

			translate([0,fst,1.5]) rotate([0,0,-90])
			front_bearings_plate();
		}

		translate([0,fst-sgp/2,-0.1])
		cylinder(h=5, r=4.25, $fn=48, center=false);

		translate([0,fs,0])
		cube(size=[65-10,dh-10,10], center=true);
	}

}

module box_back()
{
	union()
	{
		rounded_frame();

		translate([0,fst,1.5]) rotate([0,0,-90]) 2x_pillows();
	}
}

module gear_lock()
{
	union()
	{
		intersection()
		{
			cylinder(h=2, r=8.5, $fn=2*17*PI, center=false);

			translate([0,0,1])
			cube(size=[18,2*(sik-0.4),2], center=true);
		}

		cylinder(h=2+4+gtk*scl, r=sir-0.4, $fn=4*sir*PI, center=false);
	}
}

module front_bearings_plate()
{
	union()
	{
		translate([-sgp/2,0,0]) pillow_1();

		translate([-sgp/2,0,0])
		cylinder(h=phg, r=sir-0.5, $fn=4*sir*PI, center=false);

		translate([-3,0,0.5]) cube(size=[sgp-6,24,1], center=true);

		translate([ sgp/2,0,0]) pillow_2();
	}
}

module 2x_pillows()
{
	union()
	{
		translate([-sgp/2,0,0]) pillow_1();
		translate([-sgp/2,0,0])
		cylinder(h=phg, r=sir-0.5, $fn=4*sir*PI, center=false);

		translate([0,(24-1.5)/2,1]) cube(size=[sgp,1.5,2], center=true);
		translate([0,0,0.5]) cube(size=[sgp,24,1], center=true);
		translate([0,0,0.75]) cube(size=[11,10,1.5], center=true);
		translate([0,-(24-1.5)/2,1]) cube(size=[sgp,1.5,2], center=true);

		translate([ sgp/2,0,0]) pillow_1();
		translate([ sgp/2,0,0])
		cylinder(h=phg, r=sor-0.15, $fn=4*sor*PI, center=false);
	}
}

module pillow_1()
{
	union()
	{
		difference()
		{
			cylinder(h=5, r=12, $fn=4*round(20*PI/4/0.5), center=false);

			translate([0,0,1])
			cylinder(h=5, r=10.5+0.15, $fn=4*round(21*PI/4/0.5), center=false);
		}

		cylinder(h=4, r=9-0.15, $fn=4*round(18*PI/4/0.5), center=false);

		for(i=[0:4])
		{
			rotate([0,0,i*72]) translate([11/2,0,1])
			cube(size=[11,2,2], center=true);
		}

		
	}
}

module pillow_2()
{
	rd=sqrt(pow(4+5.4+1,2)+pow((9.6+0.4)/2+3+0.1,2))+1.5;

	union()
	{
		difference()
		{
			cylinder(h=5, r=rd+1.5, $fn=4*round(2*(rd+1.5)*PI/4/0.5), center=false);

			translate([0,0,1])
			cylinder(h=5, r=rd+0.3, $fn=4*round(2*(rd+0.15)*PI/4/0.5), center=false);

			cylinder(h=3, r=4, $fn=32, center=true);
		}

		for(i=[0:4])
		{
			rotate([0,0,i*72]) translate([rd+0.4-1,0,1])
			cube(size=[2,2,2], center=true);
		}
	}
}

module sharpener_drum()
{
	translate([0,24-8,0]) 2x_holder_sides();
	translate([0,-24+15,0]) holder_ends();
}

module 2x_holder_sides()
{
	translate([15,0,0]) holder_side();
	mirror([1,0,0]) translate([15,0,0]) holder_side();
}

module holder_side()
{
	a=9.6+0.4;
	b=3.4+0.1;
	c=24.9+0.1;

	d=3;
	l=33;

	x1=c/2;
	y1=a/2; y2=y1-b;

	x3=x1+1.2; x4=x1+30-c-1.2+3;
	y3=y1+3; y4=-y2+3; y5=y1+3;

	difference()
	{
		linear_extrude(height=4, convexity=10, center=false)
		polygon(points=[	[-x3,-y3],[x4,-y3],[x4,y4],[x1,y4],[-x1,y5],[-x3,y5]	],
					paths=[	[0,1,2,3,4,5]	]);

		translate([0,0,1])
		linear_extrude(height=4, convexity=10, center=false)
		polygon(points=[[-x1,-y1],[x1,-y1],[x1,-y2],[-x1,y1]], paths=[[0,1,2,3]]);

		translate([-(x1+x3)/2,0,4.2+1]) rotate([0,90,0])
		cylinder(h=2, r=4.2, $fn=4*round(8*PI/4/0.5), center=true);
	}
}


module holder_ends()
{
	rd=sqrt(pow(4+5.4+1,2)+pow((9.6+0.4)/2+3+0.1,2))+1.5;
	xs=14.4+2*(1+0.1);
	y1=9.6+0.4+2*3;
	y2=3.4+0.1+2*3;
	tr=(y1-y2)/2;

	translate([-rd-1,0,0]) holder_front_end(xs,y1,4,rd);
	translate([rd+1,0,0]) holder_back_end(xs,y2,13.8,tr,rd);
}

module holder_front_end(x,y,hh,rr)
{
	difference()
	{
		holder_end_disk(rr,hh);

		translate([2,0,1+hh/2])
		cube(size=[x,y,hh], center=true);

		translate([0,0,-0.1])
		cylinder(h=hh+0.2, r=8.5/2, $fn=4*round(8.5*PI/4/0.5), center=false);
	}
}

module holder_back_end(x,y,hg,t,rr)
{
	difference()
	{
		holder_end_disk(rr,hg);

		translate([2,-t,hg])
		cube(size=[x,y,6], center=true);

		translate([0,0,-0.1])
		fittings_1();

		translate([0,0,-0.1])
		cylinder(h=hg-4+0.1, r=sor, $fn=4*round(2*sor*PI/4/0.5), center=false);
	}
}

module holder_end_disk(rd1,t)
{
	cylinder(h=t, r=rd1, $fn=4*round(2*rd1*PI/4/0.5), center=false);
}

module spacers()
{
	c_pt = PI*sgp/sgt;
	rd1 = outer_radius(mm_per_tooth=c_pt,number_of_teeth=sgt,clearance=0.25);

	x1=rd1-0.5; x2=rd1;
	y1=1.5; y2=2;

	difference()
	{
		union()
		{
			cylinder(h=sgh, r=10.35+1, $fn=4*round(2*(10.35+1)*PI/4/res), center=false);

			fittings_2();
		}

		hole_0();

		rotate([0,0,60]) fittings_1();
	}
}

module hole_0()
{
	x1=0; x2=sor; x3=sor+0.5;
	y1=1.5; y2=2; y3=2.5;

	translate([0,0,2]) 
	rotate_extrude(convexity=10, $fn=4*round(11.1*PI/4/0.5))
	polygon(points=[	[x1,-y3],[x3,-y3],[x3,-y2],[x2,-y1],
							[x2,y1],[x3,y2],[x3,y3],[x1,y3] ],
				paths=[	[0,1,2,3,4,5,6,7]	]);
}


module fittings_1()
{
	x1=30/2;
	y1=(30/2)*tan(30);

	translate([0,0,-0.5])
	intersection()
	{
		difference()
		{
			cylinder(h=5, r=10.5, $fn=4*round(17*PI/4/res), center=false);

			translate([0,0,-0.1])
			cylinder(h=5+0.2, r=9, $fn=4*round(15*PI/4/res), center=false);
		}

		for(i=[0:2])
		{
			rotate([0,0,i*120])
			linear_extrude(height=5, convexity=10, center=false)
			polygon(points=[	[0,0],[x1,-y1],[x1,y1]	],
						paths=[	[0,1,2]	]);
		}
	}
}

module crank_bearings(xh)
{
	x1=30/2;
	y1=(30/2)*tan(31);
	fh=sgh+gtk*scl+xh;

	union()
	{
		difference()
		{
			cylinder(h=fh, r=10.35, $fn=4*round(21*PI/4/res), center=false);

			translate([0,0,-0.1])
			cylinder(h=fh+0.2, r=9.15, $fn=4*round(19*PI/4/res), center=false);

			translate([0,0,4+xh])
			for(i=[0:2])
			{
				rotate([0,0,i*120])
				linear_extrude(height=8, convexity=10, center=false)
				polygon(points=[	[0,0],[x1,-y1],[x1,y1]	],
							paths=[	[0,1,2]	]);
			}

			for(i=[0:2])
			{
				rotate([0,0,i*120-30])
				translate([0,8,(xh+2)/2])
				cube(size=[5.2,16,xh+2.4], center=true);
			}
		}

		if( stp > 0)
		{
			for(i=[0:2])
			{
				rotate([0,0,60+i*120]) translate([9.75,0,0])
				cylinder(h=stp, r=3, $fn=16, center=false);
			}
		}
	}
}

module bearings(xh)
{
	x1=30/2;
	y1=(30/2)*tan(31);
	fh=sgh+gtk*scl+xh;

		difference()
		{
			cylinder(h=fh, r=10.35, $fn=4*round(21*PI/4/res), center=false);

			translate([0,0,-0.1])
			cylinder(h=fh+0.2, r=9.15, $fn=4*round(19*PI/4/res), center=false);

		translate([0,0,4+xh])
		for(i=[0:2])
		{
			rotate([0,0,i*120])
			linear_extrude(height=8, convexity=10, center=false)
			polygon(points=[	[0,0],[x1,-y1],[x1,y1]	],
						paths=[	[0,1,2]	]);
		}
	}
}

module fittings_2()
{
	x1=30/2;
	y1=(30/2)*tan(29);
	fh=sgh+gtk*scl;

	intersection()
	{
		difference()
		{
			cylinder(h=fh, r=10.35, $fn=4*round(21*PI/4/res), center=false);

			translate([0,0,-0.1])
			cylinder(h=fh+0.2, r=9.15, $fn=4*round(19*PI/4/res), center=false);
		}

		for(i=[0:2])
		{
			rotate([0,0,i*120])
			linear_extrude(height=8, convexity=10, center=false)
			polygon(points=[	[0,0],[x1,-y1],[x1,y1]	],
						paths=[	[0,1,2]	]);
		}
	}
}

module cg_bevel()
{
	difference()
	{
		cg_bevel_gear();
		fittings_1();
	}
}

module cg_bevel_gear()
{
	x0=0;	x1=hrd;	x2=x1+0.5;	x3=x1+0.5;
	y0=3*scl/2;	y1=gtk*scl/2;	y2=y1-0.5; y3=y1-0.5;

	translate([0,0,gtk*scl/2])
	difference()
	{
		scale([scl,scl,scl])
		intersection()
		{
			translate([0,0,-gtk/2-ghg])
			scale([0.2,0.2,0.2])
			one_from_gregs_bevel_pair(	gear1_teeth = 23,
												gear2_teeth = 23,
												axis_angle = 90,
												outside_circular_pitch=1000);

			cylinder(h=gtk, r1=11.2236, r2=11.2236+gtk, center=true);
		}

		rotate_extrude(convexity=10, $fn=2*hrd*PI/0.5)
		polygon(points=[	[x0,-y0],[x3,-y0],[x3,-y1],[x1,-y3],
								[x1,y2],[x2,y1],[x2,y0],[x0,y0]	],
					paths=[	[0,1,2,3,4,5,6,7]	]);
	}
}

module one_from_gregs_bevel_pair (
	gear1_teeth = 41,
	gear2_teeth = 7,
	axis_angle = 90,
	outside_circular_pitch=1000)
{
	outside_pitch_radius1 = gear1_teeth * outside_circular_pitch / 360;
	outside_pitch_radius2 = gear2_teeth * outside_circular_pitch / 360;
	pitch_apex1=outside_pitch_radius2 * sin (axis_angle) + 
		(outside_pitch_radius2 * cos (axis_angle) + outside_pitch_radius1) / tan (axis_angle);
	cone_distance = sqrt (pow (pitch_apex1, 2) + pow (outside_pitch_radius1, 2));

	rotate([0,0,90])
	translate ([0,0,pitch_apex1+20])
	{
		bevel_gear (
			number_of_teeth=gear1_teeth,
			cone_distance=cone_distance,
			pressure_angle=30,
			outside_circular_pitch=outside_circular_pitch,
			bore_diameter=0);
	}
}

module cg_spur()
{
	c_pt = PI*sgp/sgt;
	rd1 = outer_radius(mm_per_tooth=c_pt,number_of_teeth=sgt,clearance=0.25);

	x1=rd1-0.5; x2=rd1;
	y1=1.5; y2=2;

	difference()
	{
		union()
		{
			translate([0,0,2])
			intersection()
			{
				gear (mm_per_tooth		= c_pt,
						number_of_teeth	= sgt,
						thickness			= 4,
						pressure_angle		= sgl,
						clearance			= 0.25,
						backlash				= 0.15);

				rotate_extrude(convexity=10, $fn=4*sgt*round(2*rd1*PI/4/sgt/0.5))
				polygon(points=[	[0,-y2],[x1,-y2],[x2,-y1],[x2,y1],[x1,y2],[0,y2]	],
							paths=[	[0,1,2,3,4,5]	]);
			}

			fittings_2();
		}

		hole_0();

		rotate([0,0,60]) fittings_1();
	}
}

module gear (
	mm_per_tooth    = 3,    //this is the "circular pitch"
	number_of_teeth = 11,   //total number of teeth
	thickness       = 6,    //thickness of gear in mm
	hole_diameter   = 0,    //diameter of the hole
	twist           = 0,    //teeth rotate 
	teeth_to_hide   = 0,    //number of teeth to delete
	pressure_angle  = 28,   //
	clearance       = 0.0,  //gap between top of a tooth 
	backlash        = 0.0   //gap between two meshing 
				)
{
	assign(pi = 3.1415926)
	assign(p  = mm_per_tooth * number_of_teeth / pi / 2)  //radius of pitch circle
	assign(c  = p + mm_per_tooth / pi - clearance)        //radius of outer circle
	assign(b  = p*cos(pressure_angle))                    //radius of base circle
	assign(r  = p-(c-p)-clearance)                        //radius of root circle
	assign(t  = mm_per_tooth/2-backlash/2)                //tooth thickness at pitch circle
	assign(k  = -iang(b, p) - t/2/p/pi*180)            	//angle to where involute meets base 
																			// circle on each side of tooth
	difference()
	{
		for (i = [0:number_of_teeth-teeth_to_hide-1] )
		{
			rotate([0,0,i*360/number_of_teeth])
			linear_extrude(height = thickness, center = true, convexity = 10, twist = twist)
			polygon(points=[
							[0, -hole_diameter/10],
							polar(r, -181/number_of_teeth),
							polar(r, r<b ? k : -180/number_of_teeth),
							q7(0/5,r,b,c,k, 1),q7(1/5,r,b,c,k, 1),q7(2/5,r,b,c,k, 1),
							q7(3/5,r,b,c,k, 1),q7(4/5,r,b,c,k, 1),q7(5/5,r,b,c,k, 1),
							q7(5/5,r,b,c,k,-1),q7(4/5,r,b,c,k,-1),q7(3/5,r,b,c,k,-1),
							q7(2/5,r,b,c,k,-1),q7(1/5,r,b,c,k,-1),q7(0/5,r,b,c,k,-1),
							polar(r, r<b ? -k : 180/number_of_teeth),
							polar(r, 181/number_of_teeth) ],
						paths=[[0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16] ]);
		}

		cylinder(h=2*thickness+1, r=hole_diameter/2, center=true, $fn=20);
	}
}


//these 4 functions are used by gear
function polar(r,theta)   = r*[sin(theta), cos(theta)];  //convert polar to cartesian coordinates
function iang(r1,r2)      = sqrt((r2/r1)*(r2/r1) - 1)/3.1415926*180 - acos(r1/r2); //unwind a string this many degrees to go from radius r1 to radius r2
function q7(f,r,b,r2,t,s) = q6(b,s,t,(1-f)*max(b,r)+f*r2); //radius a fraction f up the curved side of the tooth 
function q6(b,s,t,d)      = polar(d,s*(iang(b,d)+t)); //point at radius d on the involute curve

//These 5 functions let the user find the derived dimensions of the gear.
//A gear fits within a circle of radius outer_radius, and two gears should have
//their centers separated by the sum of their pictch_radius.
function circular_pitch  (mm_per_tooth=3) = mm_per_tooth;
function diametral_pitch (mm_per_tooth=3) = 3.1415926 / mm_per_tooth;
function module_value    (mm_per_tooth=3) = mm_per_tooth / pi;
function pitch_radius    (mm_per_tooth=3,number_of_teeth=11) = mm_per_tooth * number_of_teeth / 3.1415926 / 2;
function outer_radius    (mm_per_tooth=3,number_of_teeth=11,clearance=0.1)	=
													 mm_per_tooth*(1+number_of_teeth/2)/3.1415926  - clearance;              





/* *************** assembly ******************************************************* */

	p=prd*scl;
	g=gtk*scl;
	g2=g/2;
	pg=p+g;
	pg2=p+g2;

module assembly()
{
	translate([0,0,sgp/2+0.1])
	top_gears_assembly();

	translate([0,0,-sgp/2-0.1]) rotate([180,0,0])
	gear_and_drum_assemby();
}

module cross_section()
{
	difference()
	{
		assembly();
		translate([0,-50,0]) cube([100,100,200], true);
	}
}

module top_gears_assembly()
{
	rotate([90,0,0]) translate([0,0,-sok])
	import(file="gears_holder_no_stp.stl", convexity=10);



	translate([0,0,pg2]) rotate([180,0,0]) 
	import(file="bevel_gear.stl", convexity=10);

	translate([0,0,pg2+4]) rotate([180,0,0]) 
	import(file="spacers.stl", convexity=10);

	translate([0,0,pg2+12]) rotate([180,0,0])
	import(file="crank_bearing_no_stp.stl", convexity=10);

	translate([0,0,pg2+6]) rotate([180,0,0]) rotate([0,0,90])
	import(file="gear_lock.stl", convexity=10);

	translate([0,0,pg2+6+0.2]) rotate([0,0,60])
	import(file="crank_peg.stl", convexity=10);



	translate([-pg2,0,0]) rotate([0,90,0]) 
	import(file="bevel_gear.stl", convexity=10);

	translate([-pg2-4,0,0]) rotate([0,90,0]) 
	import(file="spur_gear.stl", convexity=10);

	translate([-pg2-4-9,0,0]) rotate([0,90,0]) 
	import(file="bearing.stl", convexity=10);



	translate([ pg2,0,0]) rotate([0,-90,0]) 
	import(file="bevel_gear.stl", convexity=10);

	translate([ pg2+4,0,0]) rotate([0,-90,0]) 
	import(file="spacers.stl", convexity=10);

	translate([ pg2+4+9,0,0]) rotate([0,-90,0]) 
	import(file="bearing.stl", convexity=10);
}

module gear_and_drum_assemby()
{
	translate([-pg2-4,0,0]) rotate([0,90,0]) 
	import(file="spur_gear.stl", convexity=10);

	translate([-pg2-4-9,0,0]) rotate([0,90,0]) 
	import(file="bearing.stl", convexity=10);

}

module exploded()
{
	rotate([90,0,0]) translate([0,0,-sok])
	import(file="gears_holder_no_stp.stl", convexity=10);



	translate([0,0,pg2+17]) rotate([180,0,0]) 
	import(file="bevel_gear.stl", convexity=10);

	translate([0,0,pg2+4+30]) rotate([180,0,0]) 
	import(file="spacers.stl", convexity=10);

	translate([0,0,pg2+12+43]) rotate([180,0,0])
	import(file="crank_bearing_no_stp.stl", convexity=10);

	translate([0,0,pg2+6+67]) rotate([180,0,0]) rotate([0,0,90])
	import(file="gear_lock.stl", convexity=10);

	translate([0,0,pg2+6+78]) rotate([0,0,60])
	import(file="crank_peg.stl", convexity=10);



	translate([-pg2-18,0,0]) rotate([0,90,0]) 
	import(file="bevel_gear.stl", convexity=10);

	translate([-pg2-4-33,0,0]) rotate([0,90,0]) 
	import(file="spur_gear.stl", convexity=10);

	translate([-pg2-4-9-49,0,0]) rotate([0,90,0]) 
	import(file="bearing.stl", convexity=10);



	translate([ pg2+18,0,0]) rotate([0,-90,0]) 
	import(file="bevel_gear.stl", convexity=10);

	translate([ pg2+4+33,0,0]) rotate([0,-90,0]) 
	import(file="spacers.stl", convexity=10);

	translate([ pg2+4+9+48,0,0]) rotate([0,-90,0]) 
	import(file="bearing.stl", convexity=10);
}
